@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2024 S[&]T, The Netherlands.
+ * Copyright (C) 2015-2025 S[&]T, The Netherlands.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -225,7 +225,8 @@ void harp_ingestion_done(void);
 typedef struct harp_unit_converter_struct harp_unit_converter;
 int harp_unit_converter_new(const char *from_unit, const char *to_unit, harp_unit_converter **new_unit_converter);
 void harp_unit_converter_delete(harp_unit_converter *unit_converter);
-double harp_unit_converter_convert(const harp_unit_converter *unit_converter, double value);
+float harp_unit_converter_convert_float(const harp_unit_converter *unit_converter, float value);
+double harp_unit_converter_convert_double(const harp_unit_converter *unit_converter, double value);
 int harp_unit_compare(const char *unit_a, const char *unit_b);
 int harp_unit_is_valid(const char *str);
 void harp_unit_done(void);
