@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2024 S[&]T, The Netherlands.
+ * Copyright (C) 2015-2025 S[&]T, The Netherlands.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,12 +37,16 @@
 #include <stdio.h>
 #include <string.h>
 
+#define MAX_NAME_LENGTH 128
+
 typedef enum binning_type_enum
 {
     binning_skip,
     binning_remove,
     binning_average,
-    binning_sum
+    binning_sum,
+    binning_weight,     /* only used for int32_t and float data */
+    binning_angle       /* will use averaging using 2D vectors */
 } binning_type;
 
 static binning_type get_binning_type(harp_variable *variable, harp_dimension_type dimension_type)
@@ -51,13 +55,6 @@ static binning_type get_binning_type(harp_variable *variable, harp_dimension_typ
     int num_matching_dims;
     int i;
 
-    /* remove any count/weight variables */
-    if ((variable_name_length >= 5 && strcmp(&variable->name[variable_name_length - 5], "count") == 0) ||
-        (variable_name_length >= 6 && strcmp(&variable->name[variable_name_length - 6], "weight") == 0))
-    {
-        return binning_remove;
-    }
-
     /* ensure that there is only 1 dimension of the given type */
     for (i = 0, num_matching_dims = 0; i < variable->num_dimensions; i++)
     {
@@ -88,6 +85,23 @@ static binning_type get_binning_type(harp_variable *variable, harp_dimension_typ
         }
     }
 
+    /* remove any count variables that depend on the binned dimension */
+    if (variable_name_length >= 5 && strcmp(&variable->name[variable_name_length - 5], "count") == 0)
+    {
+        return binning_remove;
+    }
+
+    /* only keep valid weight variables */
+    if (variable_name_length >= 6 && strcmp(&variable->name[variable_name_length - 6], "weight") == 0)
+    {
+        if (variable->num_dimensions < 1 || variable->dimension_type[0] != harp_dimension_time ||
+            variable->data_type != harp_type_float || variable->unit != NULL)
+        {
+            return binning_remove;
+        }
+        return binning_weight;
+    }
+
     /* we can't bin string values */
     if (variable->data_type == harp_type_string)
     {
@@ -168,6 +182,13 @@ static binning_type get_binning_type(harp_variable *variable, harp_dimension_typ
         return binning_remove;
     }
 
+    /* use unit vector based averaging for angular variables */
+    if (strstr(variable->name, "latitude") != NULL || strstr(variable->name, "longitude") != NULL ||
+        strstr(variable->name, "angle") != NULL || strstr(variable->name, "direction") != NULL)
+    {
+        return binning_angle;
+    }
+
     /* use average by default */
     return binning_average;
 }
@@ -216,6 +237,130 @@ static int filter_binnable_variables(harp_product *product, harp_dimension_type
     return 0;
 }
 
+/* find a <variable->name>_weight variable.
+ * If the variable exists but is invalid its entry in the bintype array will be set to binning_remove.
+ */
+static int get_weight_variable_for_variable(harp_product *product, harp_variable *variable, binning_type *bintype,
+                                            harp_variable **weight_variable)
+{
+    char variable_name[MAX_NAME_LENGTH];
+    int index;
+    int i;
+
+    *weight_variable = NULL;
+
+    snprintf(variable_name, MAX_NAME_LENGTH, "%s_weight", variable->name);
+    if (!harp_product_has_variable(product, variable_name))
+    {
+        return 0;
+    }
+
+    if (harp_product_get_variable_index_by_name(product, variable_name, &index) != 0)
+    {
+        return -1;
+    }
+    if (bintype[index] == binning_remove)
+    {
+        return 0;
+    }
+
+    /* make sure that the dimensions of the weight variable match the dimensions of the given variable */
+    if (product->variable[index]->num_dimensions > variable->num_dimensions)
+    {
+        bintype[index] = binning_remove;
+        return 0;
+    }
+    for (i = 0; i < product->variable[index]->num_dimensions; i++)
+    {
+        if (product->variable[index]->dimension_type[i] != variable->dimension_type[i] ||
+            product->variable[index]->dimension[i] != variable->dimension[i])
+        {
+            bintype[index] = binning_remove;
+            return 0;
+        }
+    }
+
+    *weight_variable = product->variable[index];
+
+    return 0;
+}
+
+/* get weight values for each element in the provided variable.
+ * if a '<variable>_weight' or 'weight' variable exists then 'weight' will be populated and the return value will be 1.
+ * if no applicable weight variable could be found then the return value will be 0.
+ * the return value is -1 when an error is encountered.
+ */
+static int get_weight_for_variable(harp_product *product, harp_variable *variable, binning_type *bintype, float *weight)
+{
+    harp_variable *weight_variable = NULL;
+    long i, j;
+
+    if (variable->num_dimensions <= 1 || variable->dimension_type[0] != harp_dimension_time)
+    {
+        return 0;
+    }
+
+    if (get_weight_variable_for_variable(product, variable, bintype, &weight_variable) != 0)
+    {
+        return -1;
+    }
+    if (weight_variable == NULL)
+    {
+        int index;
+
+        if (!harp_product_has_variable(product, "weight"))
+        {
+            return 0;
+        }
+        if (harp_product_get_variable_index_by_name(product, "weight", &index) != 0)
+        {
+            return -1;
+        }
+        if (bintype[index] == binning_remove)
+        {
+            return 0;
+        }
+        weight_variable = product->variable[index];
+
+        /* initial dimensions should match */
+        if (weight_variable->num_dimensions > variable->num_dimensions)
+        {
+            return 0;
+        }
+        for (i = 0; i < weight_variable->num_dimensions; i++)
+        {
+            if (weight_variable->dimension_type[i] != variable->dimension_type[i] ||
+                weight_variable->dimension[i] != variable->dimension[i])
+            {
+                return 0;
+            }
+        }
+    }
+
+    /* this should be assured by bintype[index] not being equal to binning_remove */
+    assert(weight_variable->data_type == harp_type_float);
+
+    /* store data into weight parameter */
+    if (variable->num_elements == weight_variable->num_elements)
+    {
+        memcpy(weight, weight_variable->data.float_data, weight_variable->num_elements * sizeof(float));
+    }
+    else
+    {
+        long num_sub_elements = variable->num_elements / weight_variable->num_elements;
+
+        for (i = 0; i < weight_variable->num_elements; i++)
+        {
+            for (j = 0; j < num_sub_elements; j++)
+            {
+                weight[i * num_sub_elements + j] = weight_variable->data.float_data[i];
+            }
+        }
+    }
+
+    return 1;
+}
+
 static int add_source_index(long source_index, double weight, long *cumsum_index, long **map_source_index,
                             double **map_source_weight)
 {
@@ -355,22 +500,28 @@ static int find_matching_intervals_for_bounds(harp_variable *target_bounds, harp
 
 /**
  * Rebin all variables in the product to a specified interval grid.
- * The target bounds variable should be an axis bounds variable containing the interval edges (bins) of the target grid (as 'double' values).
- * It should be a two-dimensional variable (for a time independent grid) or a three-dimensional variable (for a time dependent grid).
+ * The target bounds variable should be an axis bounds variable containing the interval edges (bins) of the target grid
+ # (as 'double' values). It should be a two-dimensional variable (for a time independent grid) or a three-dimensional
+ # variable (for a time dependent grid).
  * The last dimension should be an independent dimension of length 2 (for the lower/upper bound of each interval).
- * The dimension to use for regridding is based on the type of the second to last dimenion of the target grid variable.
- * This function cannot be used to rebin an independent dimension or rebin the time dimension.
+ * The dimension to use for regridding is based on the type of the second to last dimension of the target grid variable.
+ * This function cannot be used to rebin an independent dimension.
  *
  * For each variable in the product, a dimension-specific rule based on the variable name will determine how to rebin
  * the variable.
  * For most variables the result will be the interval weighted average of all values overlapping the target interval.
+ * This weight includes the weight of an existing weight variable (by means of multiplication) if it exists.
+ *
  * Variables that represent an integrated quantity for the rebinned dimension will use an interval weighted sum.
  * For uncertainty variables the first order propagation rules are used (assuming full correlation).
  *
- * Variables that depend on the rebinned dimenion but have no unit (or use a string data type) will be removed.
- * Any existing count or weight variables will also be removed.
+ * For angle variables a variable-specific weight will be used that contains the magnitude of the sum of the unit
+ * vectors that was used to calculate the angle average. This weight is multiplied by any existing weight variable.
  *
- * All variables that are rebinned are converted to a double data type.
+ * Variables that depend on the rebinned dimenion but have no unit (or use a string data type) will be removed.
+ * Any existing count variables that depend on the given dimension will also be removed.
+ *
+ * All variables that are rebinned (except existing weight variables) are converted to a double data type.
  * Bins that have no overlapping source boundaries will end up with a NaN value.
  *
  * \param product Product to rebin.
@@ -383,21 +534,25 @@ static int find_matching_intervals_for_bounds(harp_variable *target_bounds, harp
 LIBHARP_API int harp_product_rebin_with_axis_bounds_variable(harp_product *product, harp_variable *target_bounds)
 {
     harp_dimension_type dimension_type;
+    double nan_value = harp_nan();
     long bounds_num_time_elements = 1;
     long source_num_dim_elements;
     long target_num_dim_elements;
     harp_variable *variable;
     long variable_name_length = (long)strlen(target_bounds->name);
-    long i;
+    long weight_size = 0;
+    long i, j, k, l;
 
     /* owned memory */
     harp_variable *source_bounds = NULL;
     harp_variable *local_target_bounds = NULL;
+    binning_type *bintype = NULL;
     long *map_source_index = NULL;      /* [num_map_elements] flat index into [dim] source array */
     double *map_source_weight = NULL;   /* [num_map_elements] weight for each [dim] source array */
     long *map_offset = NULL;    /* [(time,) dim] offset index into map_source_index/map_weight for target array */
     long *map_length = NULL;    /* [(time,) dim] number of elements in map_source_index/map_weight for target array */
     double *buffer = NULL;
+    float *weight = NULL;
 
     if (variable_name_length < 7 || strcmp(&target_bounds->name[variable_name_length - 7], "_bounds") != 0)
     {
@@ -492,7 +647,7 @@ LIBHARP_API int harp_product_rebin_with_axis_bounds_variable(harp_product *produ
         }
     }
 
-    /* remove grid variables if they exists (since we don't want to rebin them) */
+    /* remove grid variables if they exist (since we don't want to rebin them) */
     /* this won't affect the source_bounds variables that we already derived */
     if (harp_product_has_variable(product, source_bounds->name))
     {
@@ -521,6 +676,27 @@ LIBHARP_API int harp_product_rebin_with_axis_bounds_variable(harp_product *produ
         }
     }
 
+    bintype = malloc(product->num_variables * sizeof(binning_type));
+    if (bintype == NULL)
+    {
+        harp_set_error(HARP_ERROR_OUT_OF_MEMORY, "out of memory (could not allocate %lu bytes) (%s:%u)",
+                       product->num_variables * sizeof(binning_type), __FILE__, __LINE__);
+        goto error;
+    }
+    for (k = 0; k < product->num_variables; k++)
+    {
+        bintype[k] = get_binning_type(product->variable[k], dimension_type);
+
+        /* determine the maximum number of elements (as size for the 'weight' array) */
+        if (bintype[k] != binning_remove && bintype[k] != binning_skip)
+        {
+            if (product->variable[k]->num_elements > weight_size)
+            {
+                weight_size = product->variable[k]->num_elements;
+            }
+        }
+    }
+
     /* allocate memory for map_offset, map_length, and buffer */
     map_offset = (long *)malloc(bounds_num_time_elements * target_num_dim_elements * (size_t)sizeof(double));
     if (map_offset == NULL)
@@ -543,6 +719,14 @@ LIBHARP_API int harp_product_rebin_with_axis_bounds_variable(harp_product *produ
                        target_num_dim_elements * sizeof(double), __FILE__, __LINE__);
         goto error;
     }
+    /* use 2x weight_size to support angular weight variables */
+    weight = (float *)malloc(2 * weight_size * (size_t)sizeof(float));
+    if (weight == NULL)
+    {
+        harp_set_error(HARP_ERROR_OUT_OF_MEMORY, "out of memory (could not allocate %lu bytes) (%s:%u)",
+                       2 * weight_size * sizeof(float), __FILE__, __LINE__);
+        goto error;
+    }
 
     /* determine which source intervals match each target interval (and associated weight) */
     if (find_matching_intervals_for_bounds(local_target_bounds, source_bounds, &map_source_index, &map_source_weight,
@@ -561,30 +745,23 @@ LIBHARP_API int harp_product_rebin_with_axis_bounds_variable(harp_product *produ
         source_num_dim_elements = target_num_dim_elements;
     }
 
-    /* regrid each variable */
-    for (i = product->num_variables - 1; i >= 0; i--)
+    /* pre-process all variables */
+    for (k = 0; k < product->num_variables; k++)
     {
-        binning_type type;
-        long target_bounds_time_index;
-        long num_blocks;
-        long num_elements;
-        long j;
-
-        variable = product->variable[i];
-
-        /* Check if we can rebin this kind of variable */
-        type = get_binning_type(variable, dimension_type);
-
-        assert(type != binning_remove);
-        if (type == binning_skip)
+        if (bintype[k] == binning_skip || bintype[k] == binning_remove)
         {
             continue;
         }
 
-        /* Ensure that the variable data consists of doubles */
-        if (variable->data_type != harp_type_double && harp_variable_convert_data_type(variable, harp_type_double) != 0)
+        variable = product->variable[k];
+
+        /* convert variables to double */
+        if (bintype[k] != binning_weight)
         {
-            goto error;
+            if (harp_variable_convert_data_type(variable, harp_type_double) != 0)
+            {
+                goto error;
+            }
         }
 
         /* Make time independent variables time dependent if source grid or target grid is 2D (i.e. time dependent) */
@@ -598,45 +775,129 @@ LIBHARP_API int harp_product_rebin_with_axis_bounds_variable(harp_product *produ
                 }
             }
         }
-        /* Also make variable time dependent if the grid dimension is time and the variable does not depend on time */
-        if (dimension_type == harp_dimension_time &&
-            (variable->num_dimensions == 0 || variable->dimension_type[0] != harp_dimension_time))
+    }
+
+    /* convert all angles to 2D vectors [cos(x),sin(x)] */
+    for (k = 0; k < product->num_variables; k++)
+    {
+        harp_variable *weight_variable = NULL;
+        long i;
+
+        if (bintype[k] != binning_angle)
         {
-            if (harp_variable_add_dimension(variable, 0, harp_dimension_time, source_num_dim_elements) != 0)
+            continue;
+        }
+
+        variable = product->variable[k];
+
+        if (get_weight_variable_for_variable(product, variable, bintype, &weight_variable) != 0)
+        {
+            goto error;
+        }
+
+        if (harp_convert_unit(variable->unit, "rad", variable->num_elements, variable->data.double_data) != 0)
+        {
+            goto error;
+        }
+        if (harp_variable_add_dimension(variable, variable->num_dimensions, harp_dimension_independent, 2) != 0)
+        {
+            goto error;
+        }
+        for (i = 0; i < variable->num_elements; i += 2)
+        {
+            double angle = variable->data.double_data[i];
+
+            if (weight_variable != NULL)
+            {
+                float norm = weight_variable->data.float_data[i / 2];
+
+                if (norm == 0 || harp_isnan(angle))
+                {
+                    variable->data.double_data[i] = 0;
+                    variable->data.double_data[i + 1] = 0;
+                    weight_variable->data.float_data[i / 2] = 0;
+                }
+                else
+                {
+                    variable->data.double_data[i] = norm * cos(angle);
+                    variable->data.double_data[i + 1] = norm * sin(angle);
+                }
+            }
+            else
+            {
+                if (harp_isnan(angle))
+                {
+                    variable->data.double_data[i] = 0;
+                    variable->data.double_data[i + 1] = 0;
+                }
+                else
+                {
+                    variable->data.double_data[i] = cos(angle);
+                    variable->data.double_data[i + 1] = sin(angle);
+                }
+            }
+        }
+    }
+
+    /* regrid each variable */
+    for (k = 0; k < product->num_variables; k++)
+    {
+        long target_bounds_time_index;
+        long num_blocks;
+        long num_elements;
+        int result = 0;
+
+        variable = product->variable[k];
+
+        if (bintype[k] == binning_skip || bintype[k] == binning_remove || bintype[k] == binning_weight)
+        {
+            continue;
+        }
+
+        if (bintype[k] != binning_angle)
+        {
+            result = get_weight_for_variable(product, variable, bintype, weight);
+            if (result < 0)
             {
                 goto error;
             }
         }
+        if (result == 0)
+        {
+            /* if there is no pre-existing weight variable then set all weights to 1 */
+            for (i = 0; i < variable->num_elements; i++)
+            {
+                weight[i] = 1;
+            }
+        }
 
-        /* treat variable as a [num_blocks, source_max_dim_elements, num_elements] array with indices [j,k,l] */
+        /* treat variable as a [num_blocks, source_max_dim_elements, num_elements] array with indices [i,j,l] */
         num_blocks = 1;
         num_elements = 1;
-        j = 0;
+        i = 0;
         assert(variable->num_dimensions > 0);
-        while (variable->dimension_type[j] != dimension_type)
+        while (variable->dimension_type[i] != dimension_type)
         {
-            assert(j < variable->num_dimensions - 1);
-            num_blocks *= variable->dimension[j];
-            j++;
+            assert(i < variable->num_dimensions - 1);
+            num_blocks *= variable->dimension[i];
+            i++;
         }
-        j++;    /* skip dimension that is going to be regridded */
-        while (j < variable->num_dimensions)
+        i++;    /* skip dimension that is going to be regridded */
+        while (i < variable->num_dimensions)
         {
-            num_elements *= variable->dimension[j];
-            j++;
+            num_elements *= variable->dimension[i];
+            i++;
         }
 
         /* rebin the data of the variable over the given dimension */
         /* keep track of time index separately since num_blocks can capture more than just the time dimension */
         target_bounds_time_index = 0;
-        for (j = 0; j < num_blocks; j++)
+        for (i = 0; i < num_blocks; i++)
         {
-            long k, l;
-
             /* keep track of time index for 2D grids */
-            if (j % (num_blocks / bounds_num_time_elements) == 0)
+            if (i % (num_blocks / bounds_num_time_elements) == 0)
             {
-                if (target_bounds->num_dimensions == 3 && j > 0)
+                if (target_bounds->num_dimensions == 3 && i > 0)
                 {
                     target_bounds_time_index++;
                 }
@@ -644,42 +905,126 @@ LIBHARP_API int harp_product_rebin_with_axis_bounds_variable(harp_product *produ
 
             for (l = 0; l < num_elements; l++)
             {
-                for (k = 0; k < target_num_dim_elements; k++)
+                for (j = 0; j < target_num_dim_elements; j++)
                 {
-                    long offset = map_offset[target_bounds_time_index * target_num_dim_elements + k];
+                    long offset = map_offset[target_bounds_time_index * target_num_dim_elements + j];
                     double weightsum = 0;
                     double valuesum = 0;
                     long m;
 
-                    for (m = 0; m < map_length[target_bounds_time_index * target_num_dim_elements + k]; m++)
+                    for (m = 0; m < map_length[target_bounds_time_index * target_num_dim_elements + j]; m++)
                     {
-                        long source_index = j * source_num_dim_elements + map_source_index[offset + m];
-                        double weight = map_source_weight[offset + m];
+                        long source_index = i * source_num_dim_elements + map_source_index[offset + m];
+                        double w = weight[source_index * num_elements + l] * map_source_weight[offset + m];
                         double value = variable->data.double_data[source_index * num_elements + l];
 
                         if (!harp_isnan(value))
                         {
-                            valuesum += weight * value;
-                            weightsum += weight;
+                            valuesum += w * value;
+                            weightsum += w;
                         }
                     }
 
                     if (weightsum != 0)
                     {
-                        buffer[k] = valuesum;
-                        if (type == binning_average)
+                        buffer[j] = valuesum;
+                        if (bintype[k] == binning_average || bintype[k] == binning_angle)
                         {
-                            buffer[k] /= weightsum;
+                            buffer[j] /= weightsum;
                         }
                     }
                     else
                     {
-                        buffer[k] = harp_nan();
+                        buffer[j] = nan_value;
                     }
                 }
-                for (k = 0; k < target_num_dim_elements; k++)
+                for (j = 0; j < target_num_dim_elements; j++)
                 {
-                    variable->data.double_data[(j * source_num_dim_elements + k) * num_elements + l] = buffer[k];
+                    variable->data.double_data[(i * source_num_dim_elements + j) * num_elements + l] = buffer[j];
+                }
+            }
+        }
+    }
+    /* do the same, but now only for the weight variables */
+    for (k = 0; k < product->num_variables; k++)
+    {
+        long target_bounds_time_index;
+        long num_blocks;
+        long num_elements;
+
+        variable = product->variable[k];
+
+        if (bintype[k] != binning_weight)
+        {
+            continue;
+        }
+
+        /* treat variable as a [num_blocks, source_max_dim_elements, num_elements] array with indices [i,j,l] */
+        num_blocks = 1;
+        num_elements = 1;
+        i = 0;
+        assert(variable->num_dimensions > 0);
+        while (variable->dimension_type[i] != dimension_type)
+        {
+            assert(i < variable->num_dimensions - 1);
+            num_blocks *= variable->dimension[i];
+            i++;
+        }
+        i++;    /* skip dimension that is going to be regridded */
+        while (i < variable->num_dimensions)
+        {
+            num_elements *= variable->dimension[i];
+            i++;
+        }
+
+        /* rebin the data of the variable over the given dimension */
+        /* keep track of time index separately since num_blocks can capture more than just the time dimension */
+        target_bounds_time_index = 0;
+        for (i = 0; i < num_blocks; i++)
+        {
+            /* keep track of time index for 2D grids */
+            if (i % (num_blocks / bounds_num_time_elements) == 0)
+            {
+                if (target_bounds->num_dimensions == 3 && i > 0)
+                {
+                    target_bounds_time_index++;
+                }
+            }
+
+            for (l = 0; l < num_elements; l++)
+            {
+                for (j = 0; j < target_num_dim_elements; j++)
+                {
+                    long offset = map_offset[target_bounds_time_index * target_num_dim_elements + j];
+                    double weightsum = 0;
+                    double valuesum = 0;
+                    long m;
+
+                    for (m = 0; m < map_length[target_bounds_time_index * target_num_dim_elements + j]; m++)
+                    {
+                        long source_index = i * source_num_dim_elements + map_source_index[offset + m];
+                        double w = map_source_weight[offset + m];
+                        double value = variable->data.float_data[source_index * num_elements + l];
+
+                        if (!harp_isnan(value))
+                        {
+                            valuesum += w * value;
+                            weightsum += w;
+                        }
+                    }
+
+                    if (weightsum != 0)
+                    {
+                        buffer[j] = valuesum / weightsum;
+                    }
+                    else
+                    {
+                        buffer[j] = 0;
+                    }
+                }
+                for (j = 0; j < target_num_dim_elements; j++)
+                {
+                    variable->data.float_data[(i * source_num_dim_elements + j) * num_elements + l] = buffer[j];
                 }
             }
         }
@@ -694,6 +1039,70 @@ LIBHARP_API int harp_product_rebin_with_axis_bounds_variable(harp_product *produ
         }
     }
 
+    /* post-process angular variables */
+    for (k = 0; k < product->num_variables; k++)
+    {
+        harp_variable *weight_variable;
+
+        variable = product->variable[k];
+
+        if (bintype[k] != binning_angle)
+        {
+            continue;
+        }
+
+        /* convert angle variables back from 2D vectors to angles */
+        for (i = 0; i < variable->num_elements; i += 2)
+        {
+            double x = variable->data.double_data[i];
+            double y = variable->data.double_data[i + 1];
+
+            weight[i / 2] = sqrt(x * x + y * y);
+            if (weight[i / 2] == 0)
+            {
+                variable->data.double_data[i] = nan_value;
+            }
+            else
+            {
+                variable->data.double_data[i] = atan2(y, x);
+            }
+        }
+        if (harp_variable_remove_dimension(variable, variable->num_dimensions - 1, 0) != 0)
+        {
+            goto error;
+        }
+        /* convert all angles back to the original unit */
+        if (harp_convert_unit("rad", variable->unit, variable->num_elements, variable->data.double_data) != 0)
+        {
+            goto error;
+        }
+
+        /* set values to NaN if weight==0, and update weight to be the norm of the averaged vector otherwise */
+        if (get_weight_variable_for_variable(product, variable, bintype, &weight_variable) != 0)
+        {
+            goto error;
+        }
+        if (weight_variable != NULL)
+        {
+            for (i = 0; i < variable->num_elements; i++)
+            {
+                weight_variable->data.float_data[i] = weight[i];
+            }
+        }
+    }
+
+    /* remove all variables that need to be removed (in reverse order!) */
+    for (k = product->num_variables - 1; k >= 0; k--)
+    {
+        if (bintype[k] == binning_remove)
+        {
+            if (harp_product_remove_variable(product, product->variable[k]) != 0)
+            {
+                goto error;
+            }
+        }
+    }
+
     /* ensure consistent axis variables in product */
     if (harp_variable_copy(target_bounds, &variable) != 0)
     {
@@ -712,7 +1121,9 @@ LIBHARP_API int harp_product_rebin_with_axis_bounds_variable(harp_product *produ
     free(map_source_weight);
     free(map_offset);
     free(map_length);
+    free(bintype);
     free(buffer);
+    free(weight);
 
     return 0;
 
@@ -735,10 +1146,18 @@ LIBHARP_API int harp_product_rebin_with_axis_bounds_variable(harp_product *produ
     {
         free(map_length);
     }
+    if (bintype != NULL)
+    {
+        free(bintype);
+    }
     if (buffer != NULL)
     {
         free(buffer);
     }
+    if (weight != NULL)
+    {
+        free(weight);
+    }
 
     return -1;
 }
